import { logger } from './logger.js'
import * as redis from './redis.js'

const cache = {}

export function set (key, value) {
  logger.trace(`[Cache] Set ${key}`)
  if (cache[key] === value) return
  cache[key] = value
  if (redis.isEnabled()) {
    return redis.client.set(key, value)
  }
}

export async function clear () {
  for (const prop of Object.getOwnPropertyNames(cache)) {
    delete cache[prop]
  }
}

export function get (key, fallback) {
  if (cache[key]) {
    logger.trace(`[Cache] Resolve "${key}" from memory`)
    return cache[key]
  }

  const promise = (async () => {
    if (redis.isEnabled()) {
      let result = await redis.client.get(key)
      if (result) {
        logger.trace(`[Cache] Resolve "${key}" from redis`)
        result = JSON.parse(result)
        cache[key] = result
        return result
      }
    }

    if (!fallback) return

    const fallbackResult = await fallback()
    if (fallbackResult) {
      logger.trace(`[Cache] Found a fallback for "${key}"`)
      cache[key] = fallbackResult
      if (redis.isEnabled()) redis.client.set(key, JSON.stringify(fallbackResult))
    }
    return fallbackResult
  })()

  cache[key] = promise

  return promise
}

export function getCache () { return cache }
